;;; Directory Local Variables
;;; For more information see (info "(emacs) Directory Variables")

((python-mode
  (python-indent-offset . 2)
  (py-continuation-offset . 4)
  (py-smart-indentation)
  (py-indent-offset . 2)
  (flycheck-pylintrc . "pylintrc")))






