{% extends "base.html" %}

{% block main %}
  <h1>Internal Server Error.</h1>

  <p>The server encountered an error and was unable to complete your
request. Either the server is overloaded or there is an error in the
application.
</p>

{% endblock %}
